# Author: Logan Tucker
# Date: 10/10/2021
# Purpose: Recreate Atari Pong

from cs1lib import *


# Define Variables
paddle1_y = 0
paddle2_y = 320

pressed_a = False
pressed_z = False
pressed_k = False
pressed_m = False

h_collision = False
v_collision_left = False
v_collision_right = False
pad_collision_left = False
pad_collision_right = False

live_play = False

new_game = False

ball_x = 200
ball_y = 200

ball_velocity_x = 5
ball_velocity_y = 5

player1_points = 0
player2_points = 0

i = 1


# Define Constants
LEFT_PAD_UP = "a"
LEFT_PAD_DOWN = "z"

RIGHT_PAD_UP = "k"
RIGHT_PAD_DOWN = "m"

SERVE = "s"

PAD_SPEED = 10

SLICE = 5

PAD_LENGTH = 80
PAD_WIDTH = 20

PAD1_X = 0
PAD2_X = 380

NEW_GAME = " "
QUIT = "q"

TOP_PAD_BOUNDARY = 0
BOTTOM_PAD_BOUNDARY = 320

TOP_BALL_BOUNDARY = 0
BOTTOM_BALL_BOUNDARY = 390

LEFT_BALL_BOUNDARY = -200  # extended to allow buffer between serves
RIGHT_BALL_BOUNDARY = 600  # extended to allow buffer between serves

BALL_STARTING_POSITION = 200
PADDLE1_STARTING_POSITION = 0
PADDLE2_STARTING_POSITION = 320

CENTER_LINE_WIDTH = 2
BALL_RADIUS = 10
SCORE_SIZE = 16
TITLE_SIZE = 100


# Purpose: Set a black background, draw a white midline, and a white score counter
# Parameters: None
def background():
    # set black background
    set_clear_color(0, 0, 0)
    clear()

    # draw midline
    enable_stroke()
    set_stroke_width(CENTER_LINE_WIDTH)
    set_stroke_color(1, 1, 1)
    draw_line(200, 0, 200, 400)

    # print each player's score
    set_font_size(SCORE_SIZE)
    draw_text(str(player1_points), 180, 380)
    draw_text(str(player2_points), 210, 380)


# Purpose: Show players the game controls before the game begins
# Parameters: None
def instructions():
    # set background to black
    set_clear_color(0, 0, 0)
    clear()

    # print the title of the game "PONG"
    set_font_size(TITLE_SIZE)
    set_stroke_color(1, 0, 0)
    draw_text("PONG", 10, 100)

    # print the game's instructions
    set_font_size(16)
    set_stroke_color(1, 1, 1)
    draw_text("instructions:", 10, 140)
    draw_text("a/b: Move left paddle up and down", 10, 160)
    draw_text("k/m: Move right paddle up and down", 10, 180)
    draw_text("s: Serve", 10, 200)
    draw_text("space bar: New game", 10, 220)
    draw_text("q: Quit the game", 10, 240)

    set_stroke_color(1, 0, 0)
    draw_text("press space bar to continue", 10, 280)


# Purpose: Draw both paddles
# Parameters: None
def paddles():
    global paddle1_y, paddle2_y

    enable_stroke()
    set_fill_color(1, 1, 1)

    # paddle 1
    set_stroke_color(1, 0, 0)
    draw_rectangle(PAD1_X, paddle1_y, PAD_WIDTH, PAD_LENGTH)

    # paddle 2
    set_stroke_color(0, 0, 1)
    draw_rectangle(PAD2_X, paddle2_y, PAD_WIDTH, PAD_LENGTH)


# Purpose: Draw a ball with a variable for both its x and y coordinates so that its location is adjustable by function
# Parameters: None
def ball():
    global ball_x, ball_y, BALL_RADIUS

    disable_stroke()
    set_fill_color(1, 1, 1)
    draw_circle(ball_x, ball_y, BALL_RADIUS)


# Purpose: Detect weather a movement key (a, z, k, or m) has been pressed
# Parameters: This function takes a "key" parameter which listens to every key I press
def keydown(key):
    global pressed_a, pressed_z, pressed_k, pressed_m, \
        LEFT_PAD_UP, LEFT_PAD_DOWN, RIGHT_PAD_UP, RIGHT_PAD_DOWN, NEW_GAME, QUIT, new_game, live_play

    # detect player 1 (left) controls
    if key == LEFT_PAD_UP:
        pressed_a = True
    if key == LEFT_PAD_DOWN:
        pressed_z = True

    # detect player 2 (right) controls
    if key == RIGHT_PAD_UP:
        pressed_k = True
    if key == RIGHT_PAD_DOWN:
        pressed_m = True

    # detect restart and new game commands
    if key == QUIT:
        cs1_quit()
    if key == NEW_GAME:
        new_game = True

    # detect the serve command
    if key == SERVE:
        live_play = True


# Purpose: Detect weather a movement key (a, z, k, or m), that was previously pressed, has been released
# Parameters: This function takes a "key" parameter which listens to every key I release after pressing
def keyup(key):
    global pressed_a, pressed_z, pressed_k, pressed_m, \
        LEFT_PAD_UP, LEFT_PAD_DOWN, RIGHT_PAD_UP, RIGHT_PAD_DOWN

    # detect player 1 (left) button release
    if key == LEFT_PAD_UP:
        pressed_a = False
    if key == LEFT_PAD_DOWN:
        pressed_z = False

    # detect player 2 (right) button release
    if key == RIGHT_PAD_UP:
        pressed_k = False
    if key == RIGHT_PAD_DOWN:
        pressed_m = False


# Purpose: Update the paddles locations based on weather or not buttons (a, z, k, or n) are pressed
# Parameters: None
def paddle_movement():
    global paddle1_y, paddle2_y, PAD_SPEED

    # use "and" statements and value conditions to stop paddles at the edge of the graphics box
    if pressed_a and paddle1_y > TOP_PAD_BOUNDARY:
        paddle1_y = paddle1_y - PAD_SPEED
    if pressed_z and paddle1_y < BOTTOM_PAD_BOUNDARY:
        paddle1_y = paddle1_y + PAD_SPEED
    if pressed_k and paddle2_y > TOP_PAD_BOUNDARY:
        paddle2_y = paddle2_y - PAD_SPEED
    if pressed_m and paddle2_y < BOTTOM_PAD_BOUNDARY:
        paddle2_y = paddle2_y + PAD_SPEED


# Purpose: Adjust the ball's velocity after each collision
# Parameters: None
def ball_movement():
    global h_collision, pad_collision_left, pad_collision_right, \
        ball_velocity_y, ball_velocity_x, ball_y, ball_x, paddle1_y, paddle2_y, \
        TOP_PAD_BOUNDARY, BOTTOM_PAD_BOUNDARY, live_play

    # ball collides with horizontal wall
    if h_collision:
        h_collision = False
        ball_velocity_y = ball_velocity_y * -1  # reverse the vertical velocity of the ball

    # ball collides with left paddle
    if pad_collision_left:
        # if the paddle is moving apply directional slice to the ball
        if pressed_a and paddle1_y > TOP_PAD_BOUNDARY:  # slice up
            ball_velocity_y = ball_velocity_y - SLICE
        if pressed_z and paddle1_y < BOTTOM_PAD_BOUNDARY:  # slice down
            ball_velocity_y = ball_velocity_y + SLICE

        pad_collision_left = False
        ball_velocity_x = ball_velocity_x * -1  # reverse the horizontal velocity of the ball

    # ball collides with right paddle
    if pad_collision_right:
        # if the paddle is moving apply directional slice to the ball
        if pressed_k and paddle2_y > TOP_PAD_BOUNDARY:  # slice up
            ball_velocity_y = ball_velocity_y - SLICE
        if pressed_m and paddle2_y < BOTTOM_PAD_BOUNDARY:  # slice down
            ball_velocity_y = ball_velocity_y + SLICE

        pad_collision_right = False
        ball_velocity_x = ball_velocity_x * -1  # reverse the horizontal speed of the ball

    # move ball
    if live_play:  # check if ball is in play before movement
        ball_x = ball_x + ball_velocity_x
        ball_y = ball_y + ball_velocity_y


# Purpose: Detect if the ball has collided with either the horizontal edges, vertical edges or paddles
# Parameters: None
def collision():
    global h_collision, v_collision_left, v_collision_right, pad_collision_left, pad_collision_right, live_play

    # ball collides with horizontal wall
    if ball_y < TOP_BALL_BOUNDARY or ball_y > BOTTOM_BALL_BOUNDARY:
        h_collision = True

    # ball collides with vertical wall
    if ball_x == LEFT_BALL_BOUNDARY:  # the boundary is extended to provide a buffer between points
        v_collision_left = True
        live_play = False
    if ball_x == RIGHT_BALL_BOUNDARY:  # the boundary is extended to provide a buffer between points
        v_collision_right = True
        live_play = False

    # ball collides with paddle
    if ball_x == 30 and paddle1_y <= ball_y <= (paddle1_y + PAD_LENGTH):
        pad_collision_left = True
    if ball_x == 370 and paddle2_y <= ball_y <= (paddle2_y + PAD_LENGTH):
        pad_collision_right = True


# Purpose: Update the score of the game after each point and game restart
# Parameters: None
def score():
    global ball_x, ball_y, v_collision_left, v_collision_right, ball_velocity_x, ball_velocity_y, \
        player1_points, player2_points, new_game, paddle1_y, paddle2_y

    # player 2 (right) scores
    if v_collision_left:
        v_collision_left = False
        player2_points = player2_points + 1

        # serve to player 1
        ball_velocity_x = -5
        ball_velocity_y = 5

        ball_x = BALL_STARTING_POSITION
        ball_y = BALL_STARTING_POSITION

    # player 1 (left) scores
    if v_collision_right:
        v_collision_right = False
        player1_points = player1_points + 1

        # serve to player 2
        ball_velocity_x = 5
        ball_velocity_y = 5

        ball_x = BALL_STARTING_POSITION
        ball_y = BALL_STARTING_POSITION

    # reset the game
    if new_game:
        new_game = False

        # reset the score
        player1_points = 0
        player2_points = 0

        # reset the paddle position
        paddle1_y = PADDLE1_STARTING_POSITION
        paddle2_y = PADDLE2_STARTING_POSITION

        # serve to player 2
        ball_velocity_x = 5
        ball_velocity_y = 5

        ball_x = BALL_STARTING_POSITION
        ball_y = BALL_STARTING_POSITION


# Purpose: Draw the pong game in its most updated state/frame
# Parameters: None
def pong():
    global new_game, i

    # show instructions at the start of the game
    if not new_game and i == 1:
        instructions()
    else:
        i = 0  # change the (i) value to prevent the instructions page from repeating
        background()  # Clear the background each time through to avoid multiple drawings of paddles and balls

        paddle_movement()
        paddles()

        collision()
        score()
        ball_movement()
        ball()


start_graphics(pong, key_press=keydown, key_release=keyup)

